
# q-expansion of newform 315.4.a.i, downloaded from the LMFDB on 09 March 2025.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 315
weight = 4
poly_data = [-16, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0, -1], [0, 0], [-5, 0], [-7, 0], [10, 2], [-12, 2], [-66, 16], [58, -14], [-140, 20], [74, 48], [54, 42], [-30, -36], [138, -100], [-156, -32], [-304, 48], [-120, -86], [464, -84], [-114, 24], [-84, 64], [-814, -42], [-92, -202], [-392, -104], [-356, -216], [-290, -8], [104, 314], [30, -28], [1032, 64], [-332, -312], [-230, 156], [-92, -342], [-1424, 128], [1020, 392], [0, 454], [602, -518], [-802, 436], [-508, 452], [-1448, -242], [964, -344], [-856, -16], [950, -140], [-1246, 594], [458, -508], [670, -94], [-3038, 288], [2732, -682], [-1074, -470], [1572, 744], [4012, -188], [-1460, 1128], [-2090, -440], [1716, 434], [-1526, -250], [-5346, 116], [2152, -844], [-1214, 1084], [-7464, 32], [-218, 228], [-3502, -354], [-4238, 1244], [7338, -52], [1964, 552], [-7210, -84], [-144, 2124], [-7368, 64], [-2680, 1826], [1208, -1054], [-1236, -696], [-3886, 1376], [-2824, 500], [-826, 896], [1934, 1232], [4430, 474], [9884, -348], [1430, -1320], [1456, -716], [416, 208], [-6442, -724], [604, -1190], [-6274, -752], [3682, 1640], [4420, -704], [-10834, -336], [10538, -698], [3832, -998], [7566, 186], [5448, -2652], [4694, -1688], [-6858, -284], [-2894, 88], [-2312, -760], [12916, -1936], [14632, -1736], [12704, -1400], [6266, 178], [-17912, -124], [-9168, -488], [-754, -1172], [-2846, 36], [-376, -868], [-6322, 2296], [11636, -616], [8272, 634], [13484, 1848], [-2762, -4576], [320, 2372], [-3088, 5202], [-1060, 5408], [-2266, -472], [1630, -1094], [7414, -4780], [6716, 1044], [-19382, -12], [12220, 1834], [-2078, 1746], [8392, 3616], [-18574, -1908], [3920, 700], [-7256, -4528], [2988, -5378], [23150, -1306], [6822, 4360], [1382, -7300], [-8170, -2708], [14904, 1764], [-22450, -290], [-25462, -176], [12638, 32], [-3692, 6908], [5660, 5660], [-3124, -2790], [-184, 916], [-8152, -16], [15848, 680], [-16746, -4008], [-9226, 5056], [19526, 4924], [-23138, 772], [-7272, 7108], [-626, 396], [8442, 5900], [1554, -3710], [-6430, 4944], [-1240, 784], [20788, -6888], [3298, -3644], [18620, -1876], [-1460, 8130], [4594, -2492], [70, -5378], [-5604, -4268], [14082, -6700], [-26274, -80], [12612, -7592], [-14112, 9032], [5212, 2280], [-43754, -1094], [88, 208], [-29154, 6240], [8564, -6010], [12218, -9232], [-39824, 4212], [24308, 4738], [43632, -2024], [18576, 924], [-28308, 5186], [13672, 3072], [-224, -4864], [26708, 810], [-6610, -2972], [-18490, -1716], [-11662, 6250], [-17406, -8348], [-33438, -3514], [498, 3432], [-70, 6806], [-262, -612], [-17324, 11024], [27334, -996], [-10492, 4740], [-30342, 5692], [-14880, -6192], [-1920, -9340], [-10042, 11992], [12942, -7720], [40632, -88], [-31362, 3636], [-35342, 2676], [-13024, -8948], [22398, -3396], [50484, 4716], [16232, 2202], [-10900, 4880], [-1168, -9700], [-13142, 3632], [-43688, -6596], [-40890, -768], [-52626, -1308], [-42954, 1168], [12882, 3180], [2440, 1616], [8602, -15472], [25518, -4862], [-192, -16882], [23566, 12116], [37520, -2212], [-16994, -420], [-18982, 2310], [-13124, 12824], [-43614, -828], [-60938, -2178], [-83434, 1820], [26486, -4356], [16512, -2600], [-9588, 704], [29196, 508], [43358, 1244], [24440, 10504], [42750, 11696], [36464, -5320], [9764, 13462], [45118, -8480], [57782, 1362], [71778, 3548], [38752, -13152], [43676, 1928], [57074, -3268], [-76414, -588], [36302, -11278], [40736, -18360], [850, 6554], [-26218, -128], [6610, 7802], [-26180, 20228], [28674, -12508], [-10776, 12156], [-42696, 1640], [6466, 752], [-27604, -410], [7202, -2022], [-38108, 12740], [-25916, -14528], [-15110, -3702], [11264, -4120], [42274, 15220], [78402, -4660], [-8444, -15372], [56108, 6436], [32204, -18408], [10620, -1160], [-21540, 20196], [-26390, 21404], [36798, -11360], [-76924, 3092], [28946, -7608], [-83434, -4364], [-21214, 17586], [-34478, 7516], [-20388, -11496], [46422, 2028], [36672, -2390], [48304, -15136], [21328, 12132], [76806, 12408], [70704, -10762], [-35154, 12976], [8358, 18870], [-2174, 11016], [-26222, 27748], [-28148, 11624], [-39356, 13822], [-79350, -6580], [-22428, -12872], [53040, -5030], [-85680, -10128], [33912, -17062], [9548, 1580], [77204, -12936], [-57306, -3328], [87114, 12432], [39928, 15388], [97624, -4424], [27736, -11680], [-83008, -5784], [-37886, 22972], [84792, 12172], [-1090, 3858], [28030, 26932], [7920, 7586], [302, -4806], [33486, 10736], [46290, 14264], [128604, -9816], [-23948, 14530], [11036, 10448], [-49794, -11912], [-36990, 1368], [-102726, -11098], [-23902, -1040], [-32864, 8284], [51552, -17972], [-37916, -6922], [-68548, 24558], [-121308, -3924], [-76104, 3628], [-33168, -15884], [-96606, 17856], [3996, 2336], [85358, -17976], [16586, 6638], [122742, 4520], [3640, -1272], [-95630, 8480], [1342, -10720], [86692, 15944], [40612, 16660], [101878, 5252], [45964, -12472], [-21562, -12358], [-5896, 20298], [14230, -328], [-88074, 3222], [39478, 4052], [75802, -13056], [8496, 22592], [82760, 10078], [82830, -5372], [-39872, 22716], [30844, -12264], [38220, 35076], [68048, -21102], [70394, -19268], [-66556, -17578], [90466, -5826], [42172, -29832], [-20148, 42216], [-5260, -16040], [3054, -10920], [-60158, 18972], [103998, 16764], [-114252, 7308], [-83874, -9456], [-3028, 14474], [-24178, -22764], [-56972, -6812], [118276, -23818], [91934, -17082], [-1434, -40520], [-56252, -12704], [-197284, -3468], [-44842, -15300], [-37238, -19950], [168830, 7964], [68122, 17360], [-85232, 4720], [-38486, 9668], [-45106, -14040], [-69556, 41172], [-84420, 18576], [100900, 9514], [-35984, -24264], [58206, 3072], [-65078, 17020], [-17368, 22552], [60958, -3618], [27016, 16196], [76742, 26420], [-1014, 31200], [-107312, 18584], [66218, -1376], [-78182, 20650], [-24082, -25274], [178120, 14648], [-147690, 15580], [43002, -23762], [-56646, 524], [-98004, 10312], [62742, -5814], [108120, -25718], [-128278, -8604], [196452, 3910], [95686, -10828], [-45900, 4754], [57824, 3992], [17672, -25162], [-57974, -21742], [114648, -11424], [-123848, -18280], [-164500, -4030], [-23092, -7320], [11584, -2416], [-58842, 27676], [66770, 8336], [-4306, -41666], [-51416, 61940], [-108886, 5262], [37290, -16672], [-42582, -4458], [-30282, -37520], [5832, -47860], [86010, 22904], [-180558, -9676], [-11348, 8370], [-105904, 13328], [-240422, 4636], [-162766, 4112], [114146, 12182], [-117958, -1316], [81238, 4184], [44884, -65784], [-68112, -20748], [-80724, -28282], [43088, -1790], [-152832, -10548], [39508, -10072], [155914, -8784], [12118, -30188], [-9814, -33930], [4876, 23036], [-68666, 8712], [107200, 21928], [-77090, 28], [-73452, 30314], [81336, 14376], [-223336, -7708], [-29368, -22830], [-101690, -5372], [26244, -33376], [27438, -504], [-7082, 11022], [-28212, 50956]]
